/* vi: set sw=4 ts=4: */
/*
 * Utility routines.
 *
 * Copyright (C) 2004 TiVo, Inc.  All rights reserved.
 * Copyright 1998 by Albert Cahalan; all rights reserved.
 * Copyright (C) 2002 by Vladimir Oleynik <dzo@simtreas.ru>
 * GNU Library General Public License Version 2, or any later version
 *
 */

#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <asm/page.h>

#include "top.h"

extern int procps_scan(int save_user_arg0, procps_status_t* tsave)
{
	static DIR *dir;
	struct dirent *entry;
	char *name;
	int n;
	char status[32];
	char buf[1024];
	FILE *fp;
	procps_status_t curstatus;
	int pid;
	long tasknice;
	struct stat sb;

	if (!dir) {
		dir = opendir("/proc");
		if(!dir)
			bb_perror_msg_and_die("Can't open /proc");
	}
	for(;;) {
		if((entry = readdir(dir)) == NULL) {
			closedir(dir);
			dir = 0;
			return 0;
		}
		name = entry->d_name;
		if (!(*name >= '0' && *name <= '9'))
			continue;

//		memset(tsave, 0, sizeof(procps_status_t));
		pid = atoi(name);
		tsave->pid = pid;

		sprintf(status, "/proc/%d/stat", pid);
		int fd;
		if((fd = open(status, O_RDONLY)) < 0)
			continue;
		ssize_t num=read(fd, buf, sizeof(buf));
		close(fd);
		if(num <= 0)
			continue;
		name = strrchr(buf, ')'); /* split into "PID (cmd" and "<rest>" */
		if(name == 0 || name[1] != ' ')
			continue;
		*name = 0;
		sscanf(buf, "%*s (%15s", tsave->short_cmd);
		n = sscanf(name+2,
		"%c "
		"%*s %*s %*s %*s %*s "     /* ppid, pgrp, session, tty, tpgid */
		"%*s %*s %*s %*s %*s " /* flags, min_flt, cmin_flt, maj_flt, cmaj_flt */
#ifdef FEATURE_CPU_USAGE_PERCENTAGE
		"%lu %lu "
#else
		"%*s %*s "
#endif
		"%*s %*s %*s "         /* cutime, cstime, priority */
		"%*s "                 /* nice */
		"%*s %*s %*s "         /* timeout, it_real_value, start_time */
		"%*s "                 /* vsize */
		"%ld",
		tsave->state,
#ifdef FEATURE_CPU_USAGE_PERCENTAGE
		&tsave->utime, &tsave->stime,
#endif
		&tsave->rss
		);
#ifdef FEATURE_CPU_USAGE_PERCENTAGE
		if(n != 4)
#else
		if(n != 2)
#endif
			continue;

		if (tsave->rss == 0 && tsave->state[0] != 'Z')
			tsave->state[1] = 'W';
		else
			tsave->state[1] = ' ';

		tsave->rss <<= (PAGE_SHIFT - 10);     /* 2**10 = 1kb */
		sprintf(status, "/proc/%d/schedstat",pid);
        if((fd = open(status, O_RDONLY)) > 0) {
        	ssize_t num=read(fd, buf, sizeof(buf));
        	close(fd);
			sscanf(buf, "%lu %lu", &tsave->rt_priority, &tsave->policy);
		} else {
			tsave->rt_priority=0;
			tsave->policy=0;
		}
		return 1;
	}
}

/* END CODE */
/*
Local Variables:
c-file-style: "linux"
c-basic-offset: 4
tab-width: 4
End:
*/
